library layout;

import 'package:flutter/material.dart';

class Expand extends Expanded {
  Expand(this.child, {this.flex = 1}) : super(child: child, flex: flex);
  final Widget child;
  final int flex;
}

class Flex extends Flexible {
  Flex(this.child, {this.flex = 1}) : super(child: child, flex: flex);
  final Widget child;
  final int flex;
}

class Cell extends StatelessWidget {
  Cell(this.content, {this.padding = 4, this.center = false});

  final dynamic content;
  final dynamic padding;
  final bool center;

  get flex => (content is DirectionalLayout) ? content.flex : 1;

  @override
  Widget build(BuildContext context) {
//    debugPrint('Building cell, ${content.toString()} | ${this.flex}');
    return Padding(
        child: center ? Center(child: content) : content,
        padding: EdgeInsets.all(padding.toDouble()));
  }
}

class UIImage extends StatelessWidget {
  UIImage(this.asset, {this.padding = 0});

  final String asset;
  final dynamic padding;

  @override
  Widget build(BuildContext context) {
    return Padding(
        child: Image.asset('assets/' + asset, fit: BoxFit.cover),
        padding: EdgeInsets.all(padding.toDouble()));
  }
}

class Direction {
  static const int Vertical = 1;
  static const int Horizontal = -1;
}

class DirectionalLayout extends StatelessWidget {
  DirectionalLayout(this.content,
      {this.direction = Direction.Vertical,
      this.flex = 1,
      this.outsidePadding = 0,
      this.insidePadding = 0,
      this.spaceEvenly = true,
      this.crossCenter = true,
      this.mainCenter = true,
      this.center = true,
      this.scrollable = true,
      this.inflate = true});

  final dynamic content;
  final int direction;
  final dynamic outsidePadding;
  final dynamic insidePadding;
  final bool spaceEvenly;
  final bool crossCenter;
  final bool mainCenter;
  final bool center;
  final bool scrollable;
  final bool inflate;
  final int flex;

  List<Widget> _processWidgets(List data) => data
          .map((x) => x is String ? UIText(x) : x)
          .map((x) => !(x is DirectionalLayout ||
                  x is MaterialButton ||
                  x is UIImage ||
                  x is Image)
              ? (scrollable ? SingleChildScrollView(child: x) : x)
              : x)
          .map(
              (x) => Cell(x, center: center, padding: insidePadding.toDouble()))
          .map<Widget>((x) {
//      debugPrint('Inflating layout item with flex ${x.flex}');
        return inflate ? Expand(x, flex: x.flex) : Flex(x, flex: x.flex);
      }).toList();

  @override
  Widget build(BuildContext context) {
    final List<Widget> _content =
        _processWidgets(content is Widget ? [content] : content);

    final CrossAxisAlignment _crossAxisAlignment = crossCenter
        ? CrossAxisAlignment.center
        : (inflate ? CrossAxisAlignment.stretch : CrossAxisAlignment.start);
    final MainAxisAlignment _mainAxisAlignment = mainCenter
        ? MainAxisAlignment.center
        : (crossCenter || spaceEvenly
            ? MainAxisAlignment.spaceAround
            : MainAxisAlignment.start);

    if (direction == Direction.Vertical) {
      return Cell(
          Column(
              mainAxisAlignment: _mainAxisAlignment,
              crossAxisAlignment: _crossAxisAlignment,
              children: _content),
          padding: outsidePadding.toDouble());
    } else if (direction == Direction.Horizontal) {
      return Cell(
          Row(
              mainAxisAlignment: _mainAxisAlignment,
              crossAxisAlignment: _crossAxisAlignment,
              children: _content),
          padding: outsidePadding.toDouble());
    }
  }
}

class Vertical extends DirectionalLayout {
  Vertical(
    this.content, {
    this.flex = 1,
    this.spaceEvenly = true,
    this.mainCenter = true,
    this.crossCenter = true,
    this.center = true,
    this.scrollable = true,
    this.inflate = true,
    this.outsidePadding = 0,
    this.insidePadding = 0,
  }) : super(
          content,
          direction: Direction.Vertical,
          flex: flex,
          spaceEvenly: spaceEvenly,
          mainCenter: mainCenter,
          crossCenter: crossCenter,
          center: center,
          scrollable: scrollable,
          inflate: inflate,
          outsidePadding: outsidePadding,
          insidePadding: insidePadding,
        );

  final dynamic content;
  final bool spaceEvenly;
  final bool mainCenter;
  final bool crossCenter;
  final bool center;
  final bool scrollable;
  final bool inflate;
  final dynamic outsidePadding;
  final dynamic insidePadding;
  final int flex;
}

class Horizontal extends DirectionalLayout {
  Horizontal(
    this.content, {
    this.flex = 1,
    this.spaceEvenly = true,
    this.mainCenter = true,
    this.crossCenter = true,
    this.center = true,
    this.scrollable = true,
    this.inflate = true,
    this.outsidePadding = 0,
    this.insidePadding = 0,
  }) : super(
          content,
          direction: Direction.Horizontal,
          flex: flex,
          spaceEvenly: spaceEvenly,
          mainCenter: mainCenter,
          crossCenter: crossCenter,
          center: center,
          scrollable: scrollable,
          inflate: inflate,
          outsidePadding: outsidePadding,
          insidePadding: insidePadding,
        );

  final dynamic content;
  final bool spaceEvenly;
  final bool mainCenter;
  final bool crossCenter;
  final bool center;
  final bool scrollable;
  final bool inflate;
  final dynamic outsidePadding;
  final dynamic insidePadding;
  final int flex;
}

class UIText extends StatelessWidget {
  UIText(this.text,
      {this.fontSize = 11,
      this.color = Colors.black,
      this.fontWeight = FontWeight.normal});

  final String text;
  final dynamic fontSize;
  final FontWeight fontWeight;
  final Color color;

  @override
  Widget build(BuildContext context) {
    return Text(
      text,
      style: TextStyle(fontSize: fontSize.toDouble(), color: color),
    );
  }
}

class Layout extends StatelessWidget {
  Layout(this.data);
  final dynamic data;

  _parseList(l) {
    if (l is! List) return l;

    if (l.length == 1 && l[0] is List) {
      return Horizontal(l[0].map(_parseList).toList());
    } else {
      return Vertical(l.map(_parseList).toList());
    }
  }

  @override
  Widget build(BuildContext context) {
    return data is List ? _parseList(data) : data;
  }
}
